﻿using Syncfusion.Pdf;
using Syncfusion.Pdf.Security;
using System;
using System.Drawing;
using System.Security.Cryptography;
using System.Security.Cryptography.Pkcs;
using System.Security.Cryptography.X509Certificates;

namespace DigitalSign
{
    class Program
    {
        static void Main(string[] args)
        {           

            //Load existing PDF document.
            PdfDocument document = new PdfDocument();

            PdfPage page = document.Pages.Add();
                      
            PdfSignature signature = new PdfSignature(document, page, null, "DigitalSignature");
            
            //Set the signature bounds.  
            signature.Bounds = new RectangleF(0, 0, 200, 100);

            //Call the compute hash event.  
            signature.ComputeHash += Signature_ComputeHash;
            //Save the PDF document.
            document.Save("WindowsStore.pdf");

            //Close the document.
            document.Close(true);

            System.Diagnostics.Process.Start("WindowsStore.pdf");
        }
        private static void Signature_ComputeHash(object sender, PdfSignatureEventArgs ars)
        {
            //Get the document bytes. 
            byte[] documentBytes = ars.Data;

            //Initialize the Windows store.
            X509Store store = new X509Store("MY", StoreLocation.CurrentUser);
            store.Open(OpenFlags.ReadOnly | OpenFlags.OpenExistingOnly);
            X509Certificate2Collection collection = (X509Certificate2Collection)store.Certificates;
            //Find the certificate using thumb print.
            X509Certificate2Collection fcollection = (X509Certificate2Collection)collection.Find(X509FindType.FindByThumbprint, "0F59645E853309589EAE9965D2E603D70B46D707", true);
            X509Certificate2 certificate = fcollection[0];

            //Include the signed data to PDF.  
            ars.SignedData = Sign(documentBytes, certificate);
        }
        public static byte[] Sign(byte[] data, X509Certificate2 certificate)
        {
            if (data == null)
                throw new ArgumentNullException("data");
            if (certificate == null)
                throw new ArgumentNullException("certificate");

            // setup the data to sign 
            ContentInfo content = new ContentInfo(data);
            SignedCms signedCms = new SignedCms(content, true);
            CmsSigner signer = new CmsSigner(certificate);
            signer.DigestAlgorithm = new Oid("SHA256");//you can use SHA1,SHA256,SHA512
            signedCms.ComputeSignature(signer);

            return signedCms.Encode();
        }
    }
}
